searchNodes=[{"ref":"EctoMorph.html","title":"EctoMorph","type":"module","doc":"Utility functions for Ecto related stuff and things. Check out the functions docs to see what is available."},{"ref":"EctoMorph.html#filter_by_schema_fields/2","title":"EctoMorph.filter_by_schema_fields/2","type":"function","doc":"Returns a map of all of the schema fields contained within data"},{"ref":"EctoMorph.html#generate_changeset/2","title":"EctoMorph.generate_changeset/2","type":"function","doc":"Casts the given data into a changeset according to the types defined by the given schema. It ignores any fields in data that are not defined in the schema, and recursively casts any embedded fields to a changeset also. Accepts a different struct as the first argument, calling Map.to_struct on it first."},{"ref":"EctoMorph.html#generate_changeset/3","title":"EctoMorph.generate_changeset/3","type":"function","doc":"Takes in a map of data and creates a changeset out of it by casting the data recursively, according to the whitelist of fields in fields. The map of data may be a struct, and the fields whitelist can whitelist fields of nested relations by providing a list for them as well. Examples If we provide a whitelist of fields, we will be passed a changeset for the changes on those fields only: ...&gt; data = %{ ...&gt; &quot;integer&quot; =&gt; &quot;77&quot;, ...&gt; &quot;steamed_hams&quot; =&gt; [%{ ...&gt; &quot;pickles&quot; =&gt; 1, ...&gt; &quot;sauce_ratio&quot; =&gt; &quot;0.7&quot;, ...&gt; &quot;double_nested_schema&quot; =&gt; %{&quot;value&quot; =&gt; &quot;works!&quot;} ...&gt; }], ...&gt; } ...&gt; EctoMorph.generate_changeset(data, SchemaUnderTest, [:integer]) ...&gt; We can also define whitelists for any arbitrarily deep relation like so: ...&gt; data = %{ ...&gt; &quot;integer&quot; =&gt; &quot;77&quot;, ...&gt; &quot;steamed_hams&quot; =&gt; [%{ ...&gt; &quot;pickles&quot; =&gt; 1, ...&gt; &quot;sauce_ratio&quot; =&gt; &quot;0.7&quot;, ...&gt; &quot;double_nested_schema&quot; =&gt; %{&quot;value&quot; =&gt; &quot;works!&quot;} ...&gt; }], ...&gt; } ...&gt; EctoMorph.generate_changeset(data, SchemaUnderTest, [ ...&gt; :integer, ...&gt; steamed_hams: [:pickles, double_nested_schema: [:value]] ...&gt; ])"},{"ref":"EctoMorph.html#into_struct/1","title":"EctoMorph.into_struct/1","type":"function","doc":"Take a changeset and returns a struct if there are no errors on the changeset. Returns an error tuple with the invalid changeset otherwise."},{"ref":"EctoMorph.html#map_from_struct/2","title":"EctoMorph.map_from_struct/2","type":"function","doc":"Creates a map out of the Ecto struct, removing the internal ecto fields. Optionally you can remove the inserted_at and updated_at timestamp fields also by passing in :exclude_timestamps as an option Examples iex&gt; map_from_struct(%Test{}, [:exclude_timestamps]) %Test{foo: &quot;bar&quot;, id: 10} iex&gt; map_from_struct(%Test{}) %Test{foo: &quot;bar&quot;, updated_at: ~N[2000-01-01 23:00:07], inserted_at: ~N[2000-01-01 23:00:07], id: 10} iex&gt; map_from_struct(%Test{}, [:exclude_timestamps, :exclude_id]) %Test{foo: &quot;bar&quot;}"},{"ref":"EctoMorph.html#to_struct/2","title":"EctoMorph.to_struct/2","type":"function","doc":"Takes some data and tries to convert it to a struct in the shape of the given schema. Casts values to the types defined by the schema dynamically using ecto changesets. Consider this: iex&gt; Jason.encode!(%{a: :b, c: Decimal.new(&quot;10&quot;)}) |&gt; Jason.decode! %{&quot;a&quot; =&gt; &quot;b&quot;, &quot;c&quot; =&gt; &quot;10&quot;} When we decode some JSON (e.g. from a jsonb column in the db or from a network request), the JSON gets decoded by our Jason lib, but not all of the information is preserved; any atom keys become strings, and if the value is a type that is not part of the JSON spec, it is casted to a string. This means we cannot pass that JSON data directly into a struct/2 function and expect a shiny Ecto struct back (struct!/2 will just raise, and struct/2 will silently return an empty struct) UNTIL NOW! Here we take care of casting the values in the json to the type that the given schema defines, as well as turning the string keys into (existing) atoms. (We know they will be existing atoms because they will exist in the schema definitions.) We filter out any keys that are not defined in the schema, and if the first argument is a struct, we call Map.from_struct/1 on it first. This can be useful for converting data between structs. Alternatively you can pass in a list of fields that you allow updates / inserts on. This list of fields can define fields for inner schemas also like so: EctoMorph.to_struct(json, SchemaUnderTest, [ :boolean, :name, :binary, :array_of_ints, steamed_hams: [:pickles, double_nested_schema: [:value]] ]) Check out the test for more full examples. Examples iex&gt; defmodule Test do ...&gt; use Ecto.Schema ...&gt; ...&gt; embedded_schema do ...&gt; field(:pageviews, :integer) ...&gt; end ...&gt; end ...&gt; {:ok, test = %Test{}} = to_struct(%{&quot;pageviews&quot; =&gt; &quot;10&quot;}, Test) ...&gt; test.pageviews 10 iex&gt; defmodule Test do ...&gt; use Ecto.Schema ...&gt; ...&gt; embedded_schema do ...&gt; field(:pageviews, :integer) ...&gt; end ...&gt; end ...&gt; json = %{&quot;pageviews&quot; =&gt; &quot;10&quot;, &quot;ignored_field&quot; =&gt; &quot;ten&quot;} ...&gt; {:ok, test = %Test{}} = to_struct(json, Test) ...&gt; test.pageviews 10"},{"ref":"EctoMorph.html#to_struct/3","title":"EctoMorph.to_struct/3","type":"function","doc":""}]